#include "snake.h"
#include "food.h"
#include "game.h"        
#include "obstacle.h"
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include "snake.h"
#include "raylib.h"

// Creates a flash effect by clearing the screen with a color for a set number of frames
void HandleFlashEffect(int *flashFrames, Color *flashColor) {
    if (*flashFrames > 0) {
        BeginDrawing();
        ClearBackground(*flashColor);
        EndDrawing();
        (*flashFrames)--;
    }
}

// Resets the game by initializing snake, food, obstacles, score, level, and state; spawns 5 initial obstacles
void ResetGameWithObstacles(Snake *snake, Food *food, Obstacle obstacles[], int *score, int *level, GameState *state) {
    InitSnake(snake);
    SpawnFood(food);
    InitObstacles(obstacles, MAX_OBSTACLES);
    *score = 0;
    *level = 1;
    *state = PLAYING;
    
    // Spawn initial obstacles
    for (int i = 0; i < 5; i++) {
        SpawnObstacle(obstacles, MAX_OBSTACLES, snake, food);
    }
}

// Main function: Starts the game, runs the game loop, and handles states (MENU, PLAYING, PAUSED, GAME_OVER)
int main(void) {
    InitWindow(SCREEN_WIDTH, SCREEN_HEIGHT, "Snake Game");
    SetTargetFPS(10);

    // Game state management
    GameState gameState = MENU;
    int flashFrames = 0;
    Color flashColor = BLANK;
    int score = 0;
    int level = 1;

    // Game components
    srand(time(NULL));
    Snake snake;
    Food food;
    Obstacle obstacles[MAX_OBSTACLES];
    
    InitSnake(&snake);
    SpawnFood(&food);
    InitObstacles(obstacles, MAX_OBSTACLES);

    while (!WindowShouldClose()) {
        // Handle flash effect first
        if (flashFrames > 0) {
            HandleFlashEffect(&flashFrames, &flashColor);
            continue;
        }

        switch (gameState) {
            case MENU:
                if (IsKeyPressed(KEY_ENTER)) {
                    ResetGameWithObstacles(&snake, &food, obstacles, &score, &gameState, &level);
                    flashColor = PURPLE;
                    flashFrames = 1;
                }
                break;

            case PLAYING:
                HandleInput(&snake);
                HandlePauseInput(&gameState);
                UpdateSnake(&snake);

                // Food collision
                if (snake.segments[0].x == food.position.x && snake.segments[0].y == food.position.y) {
                    score++;
                    snake.length++;
                    SpawnFood(&food);
                    flashColor = YELLOW;
                    flashFrames = 1;
                    
                    // Every 5 points, add a new obstacle, and level += 1
                    if (score % 5 == 0) {
                        SpawnObstacle(obstacles, MAX_OBSTACLES, &snake, &food);
                        level++;
                    }
                }

                // Collision checks
                if (CheckCollision(&snake) || CheckObstacleCollision(&snake, obstacles, MAX_OBSTACLES)) {
                    gameState = GAME_OVER;
                    flashColor = RED;
                    flashFrames = 2;
                }
                break;
            case PAUSED:
                HandlePauseInput(&gameState);
                break;

            case GAME_OVER:
                if (IsKeyPressed(KEY_ENTER)) {
                    ResetGameWithObstacles(&snake, &food, obstacles, &score, &gameState, &level);
                    flashColor = GREEN;
                    flashFrames = 1;
                }
                break;
        }
        BeginDrawing();
        // DRAWING GRIDS 
        for (int i = 0; i < SCREEN_WIDTH/CELL_SIZE; i++) {
            DrawLine(i * CELL_SIZE, 0, i * CELL_SIZE, SCREEN_HEIGHT, Fade(LIGHTGRAY, 0.2f));
        }
        for (int i = 0; i < SCREEN_HEIGHT/CELL_SIZE; i++) {
            DrawLine(0, i * CELL_SIZE, SCREEN_WIDTH, i * CELL_SIZE, Fade(LIGHTGRAY, 0.2f));
        }

        switch (gameState) {
            case MENU:
                DrawText("SNAKE GAME", SCREEN_WIDTH/2 - 150, SCREEN_HEIGHT/2 - 50, 40, GREEN);
                DrawText("Press ENTER to Start", SCREEN_WIDTH/2 - 150, SCREEN_HEIGHT/2 -1, 30, GREEN);
                break;

            case PLAYING:
                // Draw snake
                for (int i = 0; i < snake.length; i++) {
                    if (i == 0) {
                        DrawCircle(snake.segments[i].x * CELL_SIZE + CELL_SIZE/2,
                                  snake.segments[i].y * CELL_SIZE + CELL_SIZE/2,
                                  CELL_SIZE/2, DARKGREEN);
                        // Eyes for the head
                        DrawCircle(snake.segments[i].x * CELL_SIZE + CELL_SIZE/3,
                                  snake.segments[i].y * CELL_SIZE + CELL_SIZE/3,
                                  CELL_SIZE/8, YELLOW);
                        DrawCircle(snake.segments[i].x * CELL_SIZE + 2*CELL_SIZE/3,
                                  snake.segments[i].y * CELL_SIZE + CELL_SIZE/3,
                                  CELL_SIZE/8, YELLOW);
                    } 
                    // Draw snake body
                    else {
                        DrawCircle(snake.segments[i].x * CELL_SIZE + CELL_SIZE/2,
                                  snake.segments[i].y * CELL_SIZE + CELL_SIZE/2,
                                  CELL_SIZE/2 - 2, GREEN);
                    }
                }

                // Draw food
                DrawCircle(food.position.x * CELL_SIZE + CELL_SIZE/2,
                    food.position.y * CELL_SIZE + CELL_SIZE/2,
                    CELL_SIZE/3, RED);
                DrawCircle(food.position.x * CELL_SIZE + CELL_SIZE/3,
                    food.position.y * CELL_SIZE + CELL_SIZE/3,
                    CELL_SIZE/12, GREEN);  // Leaf
                            
                // Draw obstacles
                DrawObstacles(obstacles, MAX_OBSTACLES);

                // Draw score and level
                DrawText(TextFormat("Score: %d", score), 10, 10, 20, RED);
                DrawText(TextFormat("Level: %d", level), 10, 40, 20, RED);
                break;

            case PAUSED:
                for (int i = 0; i < snake.length; i++) {
                    if (i == 0) {
                        DrawCircle(snake.segments[i].x * CELL_SIZE + CELL_SIZE/2,
                                snake.segments[i].y * CELL_SIZE + CELL_SIZE/2,
                                CELL_SIZE/2, DARKGREEN);
                        // Eyes for the head
                        DrawCircle(snake.segments[i].x * CELL_SIZE + CELL_SIZE/3,
                                snake.segments[i].y * CELL_SIZE + CELL_SIZE/3,
                                CELL_SIZE/8, YELLOW);
                        DrawCircle(snake.segments[i].x * CELL_SIZE + 2*CELL_SIZE/3,
                                snake.segments[i].y * CELL_SIZE + CELL_SIZE/3,
                                CELL_SIZE/8, YELLOW);
                    } 
                    // Draw snake body
                    else {
                        DrawCircle(snake.segments[i].x * CELL_SIZE + CELL_SIZE/2,
                                snake.segments[i].y * CELL_SIZE + CELL_SIZE/2,
                                CELL_SIZE/2 - 2, GREEN);
                    }
                }        

                // Drawing apple
                DrawCircle(food.position.x * CELL_SIZE + CELL_SIZE/2,
                    food.position.y * CELL_SIZE + CELL_SIZE/2,
                    CELL_SIZE/3, RED);
                    //adding small green circle to make it look like apple
                DrawCircle(food.position.x * CELL_SIZE + CELL_SIZE/3,
                    food.position.y * CELL_SIZE + CELL_SIZE/3,
                    CELL_SIZE/12, GREEN);  // Leaf
                DrawObstacles(obstacles, MAX_OBSTACLES);

                //DRAW PAUSE OVERLAY
                DrawRectangle(0,0, SCREEN_WIDTH, SCREEN_HEIGHT, (Color){0,0,0,128});
                DrawText("Paused", SCREEN_WIDTH/2 -150, SCREEN_HEIGHT/2 -40, 50, WHITE);
                DrawText("Press ESC or P to resume", SCREEN_WIDTH/2 -150, SCREEN_HEIGHT/2 +20, 20, WHITE);
                DrawText(TextFormat("Level: %d", level), 10, 10, 20, RED);
                DrawText(TextFormat("Score: %d", score), 10, 40, 20, RED);
                break;

            case GAME_OVER:
                DrawText("GAME OVER!", SCREEN_WIDTH/2 - 150, SCREEN_HEIGHT/2 - 40, 40, RED);
                DrawText(TextFormat("Score: %d", score), SCREEN_WIDTH/2 - 120, SCREEN_HEIGHT/2 + 20, 30, RED);
                DrawText(TextFormat("Level: %d", level), SCREEN_WIDTH/2 - 120, SCREEN_HEIGHT/2 + 60, 30, RED);
                DrawText("Press ENTER to restart", SCREEN_WIDTH/2 -180, SCREEN_HEIGHT/2 + 110, 30, RED);
                break;
        }

        EndDrawing();
    }

    CloseWindow();
    return 0;
}